@@ -1,8 +1,8 @@
 /*****************************************************************************
  * gestures.c: control vlc with mouse gestures
  *****************************************************************************
- * Copyright (C) 2004 the VideoLAN team
- * $Id: 5cdffe43830c4b38e2ad16b92d62232b0f67832b $
+ * Copyright (C) 2004-2009 the VideoLAN team
+ * $Id: 5c117905c54891aaebadda02d98432b0b7584a4b $
  *
  * Authors: Sigmund Augdal Helberg <dnumgis@videolan.org>
  *
@@ -45,9 +45,10 @@
  *****************************************************************************/
 struct intf_sys_t
 {
-    vlc_object_t *      p_vout;
-    bool          b_got_gesture;
-    bool          b_button_pressed;
+    vlc_mutex_t         lock;
+    vout_thread_t      *p_vout;
+    bool                b_got_gesture;
+    bool                b_button_pressed;
     int                 i_mouse_x, i_mouse_y;
     int                 i_last_x, i_last_y;
     unsigned int        i_pattern;
@@ -68,8 +69,6 @@ struct intf_sys_t
 
 int  Open   ( vlc_object_t * );
 void Close  ( vlc_object_t * );
-static int  InitThread     ( intf_thread_t *p_intf );
-static void EndThread      ( intf_thread_t *p_intf );
 static int  MouseEvent     ( vlc_object_t *, char const *,
                              vlc_value_t, vlc_value_t, void * );
 
@@ -91,20 +90,20 @@ static const char *const button_list[] = { "left", "middle", "right" };
 static const char *const button_list_text[] =
                                    { N_("Left"), N_("Middle"), N_("Right") };
 
-vlc_module_begin();
-    set_shortname( N_("Gestures"));
-    set_category( CAT_INTERFACE );
-    set_subcategory( SUBCAT_INTERFACE_CONTROL );
+vlc_module_begin ()
+    set_shortname( N_("Gestures"))
+    set_category( CAT_INTERFACE )
+    set_subcategory( SUBCAT_INTERFACE_CONTROL )
     add_integer( "gestures-threshold", 30, NULL,
-                 THRESHOLD_TEXT, THRESHOLD_LONGTEXT, true );
+                 THRESHOLD_TEXT, THRESHOLD_LONGTEXT, true )
     add_string( "gestures-button", "right", NULL,
-                BUTTON_TEXT, BUTTON_LONGTEXT, false );
-        change_string_list( button_list, button_list_text, 0 );
-    set_description( N_("Mouse gestures control interface") );
+                BUTTON_TEXT, BUTTON_LONGTEXT, false )
+        change_string_list( button_list, button_list_text, 0 )
+    set_description( N_("Mouse gestures control interface") )
 
-    set_capability( "interface", 0 );
-    set_callbacks( Open, Close );
-vlc_module_end();
+    set_capability( "interface", 0 )
+    set_callbacks( Open, Close )
+vlc_module_end ()
 
 /*****************************************************************************
  * OpenIntf: initialize interface
@@ -114,15 +113,33 @@ int Open ( vlc_object_t *p_this )
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
 
     /* Allocate instance and initialize some members */
-    p_intf->p_sys = malloc( sizeof( intf_sys_t ) );
+    intf_sys_t *p_sys = p_intf->p_sys = malloc( sizeof( intf_sys_t ) );
     if( p_intf->p_sys == NULL )
-    {
-        return( 1 );
-    };
+        return VLC_ENOMEM;
 
+    // Configure the module
     p_intf->pf_run = RunIntf;
 
-    return( 0 );
+    vlc_mutex_init( &p_sys->lock );
+    p_sys->p_vout = NULL;
+    p_sys->b_got_gesture = false;
+    p_sys->b_button_pressed = false;
+    p_sys->i_threshold = config_GetInt( p_intf, "gestures-threshold" );
+
+    // Choose the tight button to use
+    char *psz_button = config_GetPsz( p_intf, "gestures-button" );
+    if( !strcmp( psz_button, "left" ) )
+        p_sys->i_button_mask = 1;
+    else if( !strcmp( psz_button, "middle" ) )
+        p_sys->i_button_mask = 2;
+    else // psz_button == "right"
+        p_sys->i_button_mask = 4;
+    free( psz_button );
+
+    p_sys->i_pattern = 0;
+    p_sys->i_num_gestures = 0;
+
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
@@ -133,23 +150,6 @@ static int gesture( int i_pattern, int i_num )
     return ( i_pattern >> ( i_num * 4 ) ) & 0xF;
 }
 
-/*****************************************************************************
- * input_from_playlist: don't forget to release the return value
- *  Also this function should really be available from core.
- *****************************************************************************/
-static input_thread_t * input_from_playlist ( playlist_t *p_playlist )
-{
-    input_thread_t * p_input;
-
-    PL_LOCK;
-    p_input = p_playlist->p_input;
-    if( p_input )
-        vlc_object_yield( p_input );
-    PL_UNLOCK;
-
-    return p_input;
-}
-
 /*****************************************************************************
  * CloseIntf: destroy dummy interface
  *****************************************************************************/
@@ -157,7 +157,18 @@ void Close ( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
 
+    // Destroy the callbacks
+    if( p_intf->p_sys->p_vout )
+    {
+        var_DelCallback( p_intf->p_sys->p_vout, "mouse-moved",
+                         MouseEvent, p_intf );
+        var_DelCallback( p_intf->p_sys->p_vout, "mouse-button-down",
+                         MouseEvent, p_intf );
+        vlc_object_release( p_intf->p_sys->p_vout );
+    }
+
     /* Destroy structure */
+    vlc_mutex_destroy( &p_intf->p_sys->lock );
     free( p_intf->p_sys );
 }
 
@@ -167,29 +178,20 @@ void Close ( vlc_object_t *p_this )
  *****************************************************************************/
 static void RunIntf( intf_thread_t *p_intf )
 {
+    intf_sys_t *p_sys = p_intf->p_sys;
     playlist_t * p_playlist = NULL;
-    input_thread_t* p_input;
-
-    vlc_mutex_lock( &p_intf->change_lock );
-    p_intf->p_sys->p_vout = NULL;
-    vlc_mutex_unlock( &p_intf->change_lock );
-
-    if( InitThread( p_intf ) < 0 )
-    {
-        msg_Err( p_intf, "can't initialize interface thread" );
-        return;
-    }
-    msg_Dbg( p_intf, "interface thread initialized" );
+    int canc = vlc_savecancel();
+    input_thread_t *p_input;
 
     /* Main loop */
-    while( !intf_ShouldDie( p_intf ) )
+    while( vlc_object_alive( p_intf ) )
     {
-        vlc_mutex_lock( &p_intf->change_lock );
+        vlc_mutex_lock( &p_sys->lock );
 
         /*
          * mouse cursor
          */
-        if( p_intf->p_sys->b_got_gesture )
+        if( p_sys->b_got_gesture )
         {
             vlc_value_t val;
             int i_interval = 0;
@@ -197,11 +199,11 @@ static void RunIntf( intf_thread_t *p_intf )
             /* If you modify this, please try to follow this convention:
                Start with LEFT, RIGHT for playback related commands
                and UP, DOWN, for other commands */
-            switch( p_intf->p_sys->i_pattern )
+            switch( p_sys->i_pattern )
             {
             case LEFT:
                 msg_Dbg( p_intf, "Go backward in the movie!" );
-                p_playlist = pl_Yield( p_intf );
+                p_playlist = pl_Hold( p_intf );
                 p_input = playlist_CurrentInput( p_playlist );
                 pl_Release( p_intf );
                 if( p_input )
@@ -215,9 +217,10 @@ static void RunIntf( intf_thread_t *p_intf )
                     vlc_object_release( p_input );
                 }
                 break;
+
             case RIGHT:
                 msg_Dbg( p_intf, "Go forward in the movie!" );
-                p_playlist = pl_Yield( p_intf );
+                p_playlist = pl_Hold( p_intf );
                 p_input = playlist_CurrentInput( p_playlist );
                 pl_Release( p_intf );
 
@@ -235,7 +238,7 @@ static void RunIntf( intf_thread_t *p_intf )
 
             case GESTURE(LEFT,UP,NONE,NONE):
                 msg_Dbg( p_intf, "Going slower." );
-                p_playlist = pl_Yield( p_intf );
+                p_playlist = pl_Hold( p_intf );
                 p_input = playlist_CurrentInput( p_playlist );
                 pl_Release( p_intf );
                 if( p_input )
@@ -247,7 +250,7 @@ static void RunIntf( intf_thread_t *p_intf )
 
             case GESTURE(RIGHT,UP,NONE,NONE):
                 msg_Dbg( p_intf, "Going faster." );
-                p_playlist = pl_Yield( p_intf );
+                p_playlist = pl_Hold( p_intf );
                 p_input = playlist_CurrentInput( p_playlist );
                 pl_Release( p_intf );
                 if( p_input )
@@ -259,15 +262,13 @@ static void RunIntf( intf_thread_t *p_intf )
 
             case GESTURE(LEFT,RIGHT,NONE,NONE):
             case GESTURE(RIGHT,LEFT,NONE,NONE):
+                msg_Dbg( p_intf, "Play/Pause" );
+                p_playlist = pl_Hold( p_intf );
+                p_input = playlist_CurrentInput( p_playlist );
+                pl_Release( p_intf );
+ 
+                if( p_input )
                 {
-                    input_thread_t * p_input;
-                    p_playlist = pl_Yield( p_intf );
-
-                    p_input = input_from_playlist( p_playlist );
-                    vlc_object_release( p_playlist );
-
-                    if( !p_input )
-                        break;
                     var_Get( p_input, "state", &val);
                     val.i_int = ( val.i_int != PLAYING_S ) ? PLAYING_S : PAUSE_S;
                     var_Set( p_input, "state", val);
@@ -276,17 +277,15 @@ static void RunIntf( intf_thread_t *p_intf )
                 break;
 
             case GESTURE(LEFT,DOWN,NONE,NONE):
-                p_playlist = pl_Yield( p_intf );
-
+                p_playlist = pl_Hold( p_intf );
                 playlist_Prev( p_playlist );
-                vlc_object_release( p_playlist );
+                pl_Release( p_intf );
                 break;
 
             case GESTURE(RIGHT,DOWN,NONE,NONE):
-                p_playlist = pl_Yield( p_intf );
-
+                p_playlist = pl_Hold( p_intf );
                 playlist_Next( p_playlist );
-                vlc_object_release( p_playlist );
+                pl_Release( p_intf );
                 break;
 
             case UP:
@@ -301,39 +300,38 @@ static void RunIntf( intf_thread_t *p_intf )
 
             case GESTURE(UP,DOWN,NONE,NONE):
             case GESTURE(DOWN,UP,NONE,NONE):
-                msg_Dbg(p_intf, "Mute sound");
+                msg_Dbg( p_intf, "Mute sound" );
                 aout_VolumeMute( p_intf, NULL );
                 break;
 
             case GESTURE(UP,RIGHT,NONE,NONE):
                 {
-                   input_thread_t * p_input;
-                   vlc_value_t val, list, list2;
-                   int i_count, i;
+                    vlc_value_t val, list, list2;
+                    int i_count, i;
 
-                    p_playlist = pl_Yield( p_intf );
-
-                    p_input = input_from_playlist( p_playlist );
-
-                    vlc_object_release( p_playlist );
+                    p_playlist = pl_Hold( p_intf );
+                    p_input = playlist_CurrentInput( p_playlist );
+                    pl_Release( p_intf );
 
                     if( !p_input )
                         break;
 
-                   var_Get( p_input, "audio-es", &val );
-                   var_Change( p_input, "audio-es", VLC_VAR_GETCHOICES,
-                               &list, &list2 );
-                   i_count = list.p_list->i_count;
-                   if( i_count <= 1 )
-                   {
-                       vlc_object_release( p_input );
-                       break;
-                   }
-                   for( i = 0; i < i_count; i++ )
-                   {
-                       if( val.i_int == list.p_list->p_values[i].i_int )
-                           break;
-                   }
+                    var_Get( p_input, "audio-es", &val );
+                    var_Change( p_input, "audio-es", VLC_VAR_GETCHOICES,
+                                &list, &list2 );
+                    i_count = list.p_list->i_count;
+                    if( i_count <= 1 )
+                    {
+                        var_Change( p_input, "audio-es", VLC_VAR_FREELIST, &list,
+                                    &list2 );
+                        vlc_object_release( p_input );
+                        break;
+                    }
+                    for( i = 0; i < i_count; i++ )
+                    {
+                        if( val.i_int == list.p_list->p_values[i].i_int )
+                            break;
+                    }
                     /* value of audio-es was not in choices list */
                     if( i == i_count )
                     {
@@ -346,19 +344,19 @@ static void RunIntf( intf_thread_t *p_intf )
                     else
                         i++;
                     var_Set( p_input, "audio-es", list.p_list->p_values[i] );
+                    var_Change( p_input, "audio-es", VLC_VAR_FREELIST, &list,
+                                &list2 );
                     vlc_object_release( p_input );
                 }
                 break;
             case GESTURE(DOWN,RIGHT,NONE,NONE):
                 {
-                    input_thread_t * p_input;
                     vlc_value_t val, list, list2;
                     int i_count, i;
 
-                    p_playlist = pl_Yield( p_intf );
-
-                    p_input = input_from_playlist( p_playlist );
-                    vlc_object_release( p_playlist );
+                    p_playlist = pl_Hold( p_intf );
+                    p_input = playlist_CurrentInput( p_playlist );
+                    pl_Release( p_intf );
 
                     if( !p_input )
                         break;
@@ -371,6 +369,8 @@ static void RunIntf( intf_thread_t *p_intf )
                     if( i_count <= 1 )
                     {
                         vlc_object_release( p_input );
+                        var_Change( p_input, "spu-es", VLC_VAR_FREELIST,
+                                    &list, &list2 );
                         break;
                     }
                     for( i = 0; i < i_count; i++ )
@@ -392,131 +392,76 @@ static void RunIntf( intf_thread_t *p_intf )
                     else
                         i++;
                     var_Set( p_input, "spu-es", list.p_list->p_values[i] );
+                    var_Change( p_input, "spu-es", VLC_VAR_FREELIST,
+                                &list, &list2 );
                     vlc_object_release( p_input );
                 }
                 break;
+
             case GESTURE(UP,LEFT,NONE,NONE):
-                if (p_intf->p_sys->p_vout )
+                if( p_sys->p_vout )
                 {
-                    ((vout_thread_t *)p_intf->p_sys->p_vout)->i_changes |=
-                        VOUT_FULLSCREEN_CHANGE;
+                    var_Get( p_sys->p_vout, "fullscreen", &val );
+                    val.b_bool = !val.b_bool;
+                    var_Set( p_sys->p_vout, "fullscreen", val );
                 }
                 break;
+
             case GESTURE(DOWN,LEFT,NONE,NONE):
                 /* FIXME: Should close the vout!"*/
-                vlc_object_kill( p_intf->p_libvlc );
+                libvlc_Quit( p_intf->p_libvlc );
                 break;
             case GESTURE(DOWN,LEFT,UP,RIGHT):
             case GESTURE(UP,RIGHT,DOWN,LEFT):
-                msg_Dbg(p_intf, "a square was drawn!" );
+                msg_Dbg( p_intf, "a square was drawn!" );
                 break;
             default:
                 break;
             }
-            p_intf->p_sys->i_num_gestures = 0;
-            p_intf->p_sys->i_pattern = 0;
-            p_intf->p_sys->b_got_gesture = false;
+            p_sys->i_num_gestures = 0;
+            p_sys->i_pattern = 0;
+            p_sys->b_got_gesture = false;
         }
 
         /*
          * video output
          */
-        if( p_intf->p_sys->p_vout && !vlc_object_alive (p_intf->p_sys->p_vout) )
+        if( p_sys->p_vout && !vlc_object_alive( p_sys->p_vout ) )
         {
-            var_DelCallback( p_intf->p_sys->p_vout, "mouse-moved",
+            var_DelCallback( p_sys->p_vout, "mouse-moved",
                              MouseEvent, p_intf );
-            var_DelCallback( p_intf->p_sys->p_vout, "mouse-button-down",
+            var_DelCallback( p_sys->p_vout, "mouse-button-down",
                              MouseEvent, p_intf );
-            vlc_object_release( p_intf->p_sys->p_vout );
-            p_intf->p_sys->p_vout = NULL;
+            vlc_object_release( p_sys->p_vout );
+            p_sys->p_vout = NULL;
         }
 
-        if( p_intf->p_sys->p_vout == NULL )
+        if( p_sys->p_vout == NULL )
         {
-            p_intf->p_sys->p_vout = vlc_object_find( p_intf,
-                                      VLC_OBJECT_VOUT, FIND_ANYWHERE );
-            if( p_intf->p_sys->p_vout )
+            p_playlist = pl_Hold( p_intf );
+            p_input = playlist_CurrentInput( p_playlist );
+            pl_Release( p_intf );
+            if( p_input )
             {
-                var_AddCallback( p_intf->p_sys->p_vout, "mouse-moved",
+                p_sys->p_vout = input_GetVout( p_input );
+                vlc_object_release( p_input );
+            }
+            if( p_sys->p_vout )
+            {
+                var_AddCallback( p_sys->p_vout, "mouse-moved",
                                  MouseEvent, p_intf );
-                var_AddCallback( p_intf->p_sys->p_vout, "mouse-button-down",
+                var_AddCallback( p_sys->p_vout, "mouse-button-down",
                                  MouseEvent, p_intf );
             }
         }
 
-        vlc_mutex_unlock( &p_intf->change_lock );
+        vlc_mutex_unlock( &p_sys->lock );
 
         /* Wait a bit */
         msleep( INTF_IDLE_SLEEP );
     }
 
-    EndThread( p_intf );
-}
-
-/*****************************************************************************
- * InitThread:
- *****************************************************************************/
-static int InitThread( intf_thread_t * p_intf )
-{
-    char *psz_button;
-    /* we might need some locking here */
-    if( !intf_ShouldDie( p_intf ) )
-    {
-        /* p_intf->change_lock locking strategy:
-         * - Every access to p_intf->p_sys are locked threw p_intf->change_lock
-         * - make sure there won't be  cross increment/decrement ref count
-         *   of p_intf->p_sys members p_intf->change_lock should be locked
-         *   during those operations */
-        vlc_mutex_lock( &p_intf->change_lock );
-
-        p_intf->p_sys->b_got_gesture = false;
-        p_intf->p_sys->b_button_pressed = false;
-        p_intf->p_sys->i_threshold =
-                     config_GetInt( p_intf, "gestures-threshold" );
-        psz_button = config_GetPsz( p_intf, "gestures-button" );
-        if ( !strcmp( psz_button, "left" ) )
-        {
-            p_intf->p_sys->i_button_mask = 1;
-        }
-        else if ( !strcmp( psz_button, "middle" ) )
-        {
-            p_intf->p_sys->i_button_mask = 2;
-        }
-        else if ( !strcmp( psz_button, "right" ) )
-        {
-            p_intf->p_sys->i_button_mask = 4;
-        }
-        free( psz_button );
-
-        p_intf->p_sys->i_pattern = 0;
-        p_intf->p_sys->i_num_gestures = 0;
-        vlc_mutex_unlock( &p_intf->change_lock );
-
-        return 0;
-    }
-    else
-    {
-        return -1;
-    }
-}
-
-/*****************************************************************************
- * EndThread:
- *****************************************************************************/
-static void EndThread( intf_thread_t * p_intf )
-{
-    vlc_mutex_lock( &p_intf->change_lock );
-
-    if( p_intf->p_sys->p_vout )
-    {
-        var_DelCallback( p_intf->p_sys->p_vout, "mouse-moved",
-                         MouseEvent, p_intf );
-        var_DelCallback( p_intf->p_sys->p_vout, "mouse-button-down",
-                         MouseEvent, p_intf );
-        vlc_object_release( p_intf->p_sys->p_vout );
-    }
-
-    vlc_mutex_unlock( &p_intf->change_lock );
+    vlc_restorecancel( canc );
 }
 
 /*****************************************************************************
@@ -526,33 +471,29 @@ static int MouseEvent( vlc_object_t *p_this, char const *psz_var,
                        vlc_value_t oldval, vlc_value_t newval, void *p_data )
 {
     VLC_UNUSED(p_this); VLC_UNUSED(oldval);
-    vlc_value_t val;
     int pattern = 0;
 
     signed int i_horizontal, i_vertical;
     intf_thread_t *p_intf = (intf_thread_t *)p_data;
+    intf_sys_t    *p_sys = p_intf->p_sys;
 
-    vlc_mutex_lock( &p_intf->change_lock );
+    vlc_mutex_lock( &p_sys->lock );
 
     /* don't process new gestures before the last events are processed */
-    if( p_intf->p_sys->b_got_gesture )
+    if( p_sys->b_got_gesture )
     {
-        vlc_mutex_unlock( &p_intf->change_lock );
+        vlc_mutex_unlock( &p_sys->lock );
         return VLC_SUCCESS;
     }
 
-    if( !strcmp(psz_var, "mouse-moved" ) && p_intf->p_sys->b_button_pressed )
+    if( !strcmp( psz_var, "mouse-moved" ) && p_sys->b_button_pressed )
     {
-        var_Get( p_intf->p_sys->p_vout, "mouse-x", &val );
-        p_intf->p_sys->i_mouse_x = val.i_int;
-        var_Get( p_intf->p_sys->p_vout, "mouse-y", &val );
-        p_intf->p_sys->i_mouse_y = val.i_int;
-        i_horizontal = p_intf->p_sys->i_mouse_x -
-            p_intf->p_sys->i_last_x;
-        i_horizontal = i_horizontal / p_intf->p_sys->i_threshold;
-        i_vertical = p_intf->p_sys->i_mouse_y
-            - p_intf->p_sys->i_last_y;
-        i_vertical = i_vertical / p_intf->p_sys->i_threshold;
+        p_sys->i_mouse_x = var_GetInteger( p_sys->p_vout, "mouse-x" );
+        p_sys->i_mouse_y = var_GetInteger( p_sys->p_vout, "mouse-y" );
+        i_horizontal = p_sys->i_mouse_x - p_sys->i_last_x;
+        i_horizontal = i_horizontal / p_sys->i_threshold;
+        i_vertical = p_sys->i_mouse_y - p_sys->i_last_y;
+        i_vertical = i_vertical / p_sys->i_threshold;
 
         if( i_horizontal < 0 )
         {
@@ -576,37 +517,33 @@ static int MouseEvent( vlc_object_t *p_this, char const *psz_var,
         }
         if( pattern )
         {
-            p_intf->p_sys->i_last_y = p_intf->p_sys->i_mouse_y;
-            p_intf->p_sys->i_last_x = p_intf->p_sys->i_mouse_x;
-            if( gesture( p_intf->p_sys->i_pattern,
-                         p_intf->p_sys->i_num_gestures - 1 ) != pattern )
+            p_sys->i_last_y = p_sys->i_mouse_y;
+            p_sys->i_last_x = p_sys->i_mouse_x;
+            if( gesture( p_sys->i_pattern, p_sys->i_num_gestures - 1 )
+                    != pattern )
             {
-                p_intf->p_sys->i_pattern |=
-                    pattern << ( p_intf->p_sys->i_num_gestures * 4 );
-                p_intf->p_sys->i_num_gestures++;
+                p_sys->i_pattern |= pattern << ( p_sys->i_num_gestures * 4 );
+                p_sys->i_num_gestures++;
             }
         }
 
     }
-    if( !strcmp( psz_var, "mouse-button-down" )
-        && newval.i_int & p_intf->p_sys->i_button_mask
-        && !p_intf->p_sys->b_button_pressed )
+    else if( !strcmp( psz_var, "mouse-button-down" ) )
     {
-        p_intf->p_sys->b_button_pressed = true;
-        var_Get( p_intf->p_sys->p_vout, "mouse-x", &val );
-        p_intf->p_sys->i_last_x = val.i_int;
-        var_Get( p_intf->p_sys->p_vout, "mouse-y", &val );
-        p_intf->p_sys->i_last_y = val.i_int;
-    }
-    if( !strcmp( psz_var, "mouse-button-down" )
-        && !( newval.i_int & p_intf->p_sys->i_button_mask )
-        && p_intf->p_sys->b_button_pressed )
-    {
-        p_intf->p_sys->b_button_pressed = false;
-        p_intf->p_sys->b_got_gesture = true;
+        if( (newval.i_int & p_sys->i_button_mask) && !p_sys->b_button_pressed )
+        {
+            p_sys->b_button_pressed = true;
+            p_sys->i_last_x = var_GetInteger( p_sys->p_vout, "mouse-x" );
+            p_sys->i_last_y = var_GetInteger( p_sys->p_vout, "mouse-y" );
+        }
+        else if( !( newval.i_int & p_sys->i_button_mask ) && p_sys->b_button_pressed )
+        {
+            p_sys->b_button_pressed = false;
+            p_sys->b_got_gesture = true;
+        }
     }
 
-    vlc_mutex_unlock( &p_intf->change_lock );
+    vlc_mutex_unlock( &p_sys->lock );
 
     return VLC_SUCCESS;
 }
