@@ -2,7 +2,7 @@
  * infopanels.cpp : Panels for the information dialogs
  ****************************************************************************
  * Copyright (C) 2006-2007 the VideoLAN team
- * $Id: dcd0417371e93138c75421b48f39f62bfea7ec9f $
+ * $Id: 98bd70791b2512545f3289eb482e05472bc35879 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -422,7 +422,10 @@ InfoPanel::InfoPanel( QWidget *parent,
 
      InfoTree = new QTreeWidget(this);
      InfoTree->setColumnCount( 1 );
+     InfoTree->setColumnWidth( 0, 20000 );
      InfoTree->header()->hide();
+//     InfoTree->header()->setStretchLastSection(false);
+//     InfoTree->header()->setResizeMode(QHeaderView::ResizeToContents);
      layout->addWidget(InfoTree, 1, 0 );
 }
 
