@@ -1,8 +1,8 @@
 /*****************************************************************************
  * logger.c : file logging plugin for vlc
  *****************************************************************************
- * Copyright (C) 2002 the VideoLAN team
- * $Id: 7bdefc164bf71263aa0d907b33db949544856aa5 $
+ * Copyright (C) 2002-2008 the VideoLAN team
+ * $Id: 88d49624b89f64015f220b89adee23cfa1458083 $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -35,7 +35,7 @@
 #include <vlc_playlist.h>
 #include <vlc_charset.h>
 
-#include <errno.h>                                                 /* ENOMEM */
+#include <assert.h>
 
 #ifdef UNDER_CE
 #   define _IONBF 0x0004
@@ -52,8 +52,6 @@
 #define LOG_FILE_TEXT "vlc-log.txt"
 #define LOG_FILE_HTML "vlc-log.html"
 
-#define LOG_STRING( msg, file ) fwrite( msg, strlen( msg ), 1, file );
-
 #define TEXT_HEADER "-- logger module started --\n"
 #define TEXT_FOOTER "-- logger module stopped --\n"
 
@@ -78,18 +76,20 @@
 #include <syslog.h>
 #endif
 
+struct msg_cb_data_t
+{
+    intf_thread_t *p_intf;
+    FILE *p_file;
+    int   i_mode;
+};
+
 /*****************************************************************************
  * intf_sys_t: description and status of log interface
  *****************************************************************************/
 struct intf_sys_t
 {
-    int i_mode;
-    FILE *p_rrd;
-    mtime_t last_update;
-    time_t now;  /* timestamp for rrd-log */
-
-    FILE *    p_file; /* The log file */
     msg_subscription_t *p_sub;
+    msg_cb_data_t msg;
 };
 
 /*****************************************************************************
@@ -97,17 +97,14 @@ struct intf_sys_t
  *****************************************************************************/
 static int  Open    ( vlc_object_t * );
 static void Close   ( vlc_object_t * );
-static void Run     ( intf_thread_t * );
 
-static void FlushQueue        ( msg_subscription_t *, FILE *, int, int );
+static void Overflow (msg_cb_data_t *p_sys, msg_item_t *p_item, unsigned overruns);
 static void TextPrint         ( const msg_item_t *, FILE * );
 static void HtmlPrint         ( const msg_item_t *, FILE * );
 #ifdef HAVE_SYSLOG_H
 static void SyslogPrint       ( const msg_item_t *);
 #endif
 
-static void DoRRD( intf_thread_t *p_intf );
-
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
@@ -132,26 +129,24 @@ static const char *const mode_list_text[] = { N_("Text"), "HTML"
   "\"text\" (default) and \"html\".")
 #endif
 
-vlc_module_begin();
-    set_shortname( N_( "Logging" ) );
-    set_description( N_("File logging") );
+vlc_module_begin ()
+    set_shortname( N_( "Logging" ) )
+    set_description( N_("File logging") )
 
-    set_category( CAT_ADVANCED );
-    set_subcategory( SUBCAT_ADVANCED_MISC );
+    set_category( CAT_ADVANCED )
+    set_subcategory( SUBCAT_ADVANCED_MISC )
 
     add_file( "logfile", NULL, NULL,
-             N_("Log filename"), N_("Specify the log filename."), false );
-        change_unsafe();
+             N_("Log filename"), N_("Specify the log filename."), false )
     add_string( "logmode", "text", NULL, LOGMODE_TEXT, LOGMODE_LONGTEXT,
-                false );
-        change_string_list( mode_list, mode_list_text, 0 );
+                false )
+        change_string_list( mode_list, mode_list_text, 0 )
 
-    add_file( "rrd-file", NULL, NULL, N_("RRD output file") ,
-                    N_("Output data for RRDTool in this file." ), true );
+    add_obsolete_string( "rrd-file" )
 
-    set_capability( "interface", 0 );
-    set_callbacks( Open, Close );
-vlc_module_end();
+    set_capability( "interface", 0 )
+    set_callbacks( Open, Close )
+vlc_module_end ()
 
 /*****************************************************************************
  * Open: initialize and create stuff
@@ -159,59 +154,58 @@ vlc_module_end();
 static int Open( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
-    char *psz_mode, *psz_file, *psz_rrd_file;
+    intf_sys_t *p_sys;
+    char *psz_mode;
 
     CONSOLE_INTRO_MSG;
     msg_Info( p_intf, "using logger..." );
 
     /* Allocate instance and initialize some members */
-    p_intf->p_sys = (intf_sys_t *)malloc( sizeof( intf_sys_t ) );
-    if( p_intf->p_sys == NULL )
-        return -1;
+    p_sys = p_intf->p_sys = (intf_sys_t *)malloc( sizeof( intf_sys_t ) );
+    if( p_sys == NULL )
+        return VLC_ENOMEM;
 
+    p_sys->msg.p_intf = p_intf;
+    p_sys->msg.i_mode = MODE_TEXT;
     psz_mode = var_CreateGetString( p_intf, "logmode" );
     if( psz_mode )
     {
         if( !strcmp( psz_mode, "text" ) )
-        {
-            p_intf->p_sys->i_mode = MODE_TEXT;
-        }
+            ;
         else if( !strcmp( psz_mode, "html" ) )
         {
-            p_intf->p_sys->i_mode = MODE_HTML;
+            p_sys->msg.i_mode = MODE_HTML;
         }
 #ifdef HAVE_SYSLOG_H
         else if( !strcmp( psz_mode, "syslog" ) )
         {
-            p_intf->p_sys->i_mode = MODE_SYSLOG;
+            p_sys->msg.i_mode = MODE_SYSLOG;
         }
 #endif
         else
         {
             msg_Warn( p_intf, "invalid log mode `%s', using `text'", psz_mode );
-            p_intf->p_sys->i_mode = MODE_TEXT;
+            p_sys->msg.i_mode = MODE_TEXT;
         }
-
         free( psz_mode );
     }
     else
     {
         msg_Warn( p_intf, "no log mode specified, using `text'" );
-        p_intf->p_sys->i_mode = MODE_TEXT;
     }
 
-    if( p_intf->p_sys->i_mode != MODE_SYSLOG )
+    if( p_sys->msg.i_mode != MODE_SYSLOG )
     {
-        psz_file = config_GetPsz( p_intf, "logfile" );
+        char *psz_file = config_GetPsz( p_intf, "logfile" );
         if( !psz_file )
         {
 #ifdef __APPLE__
             if( asprintf( &psz_file, "%s/"LOG_DIR"/%s", config_GetHomeDir(),
-                (p_intf->p_sys->i_mode == MODE_HTML) ? LOG_FILE_HTML
-                                                     : LOG_FILE_TEXT ) == -1 )
+                (p_sys->msg.i_mode == MODE_HTML) ? LOG_FILE_HTML
+                                             : LOG_FILE_TEXT ) == -1 )
                 psz_file = NULL;
 #else
-            switch( p_intf->p_sys->i_mode )
+            switch( p_sys->msg.i_mode )
             {
             case MODE_HTML:
                 psz_file = strdup( LOG_FILE_HTML );
@@ -228,50 +222,39 @@ static int Open( vlc_object_t *p_this )
 
         /* Open the log file and remove any buffering for the stream */
         msg_Dbg( p_intf, "opening logfile `%s'", psz_file );
-        p_intf->p_sys->p_file = utf8_fopen( psz_file, "at" );
-        if( p_intf->p_sys->p_file == NULL )
+        p_sys->msg.p_file = utf8_fopen( psz_file, "at" );
+        if( p_sys->msg.p_file == NULL )
         {
             msg_Err( p_intf, "error opening logfile `%s'", psz_file );
-            free( p_intf->p_sys );
+            free( p_sys );
             free( psz_file );
             return -1;
         }
-        setvbuf( p_intf->p_sys->p_file, NULL, _IONBF, 0 );
+        setvbuf( p_sys->msg.p_file, NULL, _IONBF, 0 );
 
         free( psz_file );
 
-        switch( p_intf->p_sys->i_mode )
+        switch( p_sys->msg.i_mode )
         {
         case MODE_HTML:
-            LOG_STRING( HTML_HEADER, p_intf->p_sys->p_file );
+            fputs( HTML_HEADER, p_sys->msg.p_file );
             break;
         case MODE_TEXT:
         default:
-            LOG_STRING( TEXT_HEADER, p_intf->p_sys->p_file );
+            fputs( TEXT_HEADER, p_sys->msg.p_file );
             break;
         }
 
     }
     else
     {
-        p_intf->p_sys->p_file = NULL;
+        p_sys->msg.p_file = NULL;
 #ifdef HAVE_SYSLOG_H
         openlog( "vlc", LOG_PID|LOG_NDELAY, LOG_DAEMON );
 #endif
     }
 
-    p_intf->p_sys->last_update = 0;
-    p_intf->p_sys->p_rrd = NULL;
-
-    psz_rrd_file = config_GetPsz( p_intf, "rrd-file" );
-    if( psz_rrd_file && *psz_rrd_file )
-    {
-        p_intf->p_sys->p_rrd = utf8_fopen( psz_rrd_file, "w" );
-    }
-    free( psz_rrd_file );
-
-    p_intf->p_sys->p_sub = msg_Subscribe( p_intf );
-    p_intf->pf_run = Run;
+    p_sys->p_sub = msg_Subscribe( p_intf->p_libvlc, Overflow, &p_sys->msg );
 
     return 0;
 }
@@ -282,124 +265,80 @@ static int Open( vlc_object_t *p_this )
 static void Close( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
+    intf_sys_t *p_sys = p_intf->p_sys;
 
     /* Flush the queue and unsubscribe from the message queue */
-    FlushQueue( p_intf->p_sys->p_sub, p_intf->p_sys->p_file,
-                p_intf->p_sys->i_mode,
-                var_CreateGetInteger( p_intf, "verbose" ) );
-    msg_Unsubscribe( p_intf, p_intf->p_sys->p_sub );
+    /* FIXME: flush */
+    msg_Unsubscribe( p_sys->p_sub );
 
-    switch( p_intf->p_sys->i_mode )
+    switch( p_sys->msg.i_mode )
     {
     case MODE_HTML:
-        LOG_STRING( HTML_FOOTER, p_intf->p_sys->p_file );
+        fputs( HTML_FOOTER, p_sys->msg.p_file );
         break;
-    case MODE_TEXT:
 #ifdef HAVE_SYSLOG_H
     case MODE_SYSLOG:
         closelog();
         break;
 #endif
+    case MODE_TEXT:
     default:
-        LOG_STRING( TEXT_FOOTER, p_intf->p_sys->p_file );
+        fputs( TEXT_FOOTER, p_sys->msg.p_file );
         break;
     }
 
     /* Close the log file */
-    if( p_intf->p_sys->i_mode != MODE_SYSLOG )
-        fclose( p_intf->p_sys->p_file );
+    if( p_sys->msg.p_file )
+        fclose( p_sys->msg.p_file );
 
     /* Destroy structure */
-    free( p_intf->p_sys );
+    free( p_sys );
 }
 
-/*****************************************************************************
- * Run: rc thread
- *****************************************************************************
- * This part of the interface is in a separate thread so that we can call
- * exec() from within it without annoying the rest of the program.
- *****************************************************************************/
-static void Run( intf_thread_t *p_intf )
+/**
+ * Log a message
+ */
+static void Overflow (msg_cb_data_t *p_sys, msg_item_t *p_item, unsigned overruns)
 {
-    while( vlc_object_alive (p_intf) )
-    {
-        FlushQueue( p_intf->p_sys->p_sub, p_intf->p_sys->p_file,
-                    p_intf->p_sys->i_mode,
-                    var_CreateGetInteger( p_intf, "verbose" ) );
-        if( p_intf->p_sys->p_rrd )
-            DoRRD( p_intf );
+    int verbosity = var_CreateGetInteger( p_sys->p_intf, "verbose" );
+    int priority = 0;
 
-        msleep( INTF_IDLE_SLEEP );
+    switch( p_item->i_type )
+    {
+        case VLC_MSG_WARN: priority = 1; break;
+        case VLC_MSG_DBG:  priority = 2; break;
     }
-}
+    if (verbosity < priority)
+        return;
 
-/*****************************************************************************
- * FlushQueue: flush the message queue into the log
- *****************************************************************************/
-static void FlushQueue( msg_subscription_t *p_sub, FILE *p_file, int i_mode,
-                        int i_verbose )
-{
-    int i_start, i_stop;
-
-    vlc_mutex_lock( p_sub->p_lock );
-    i_stop = *p_sub->pi_stop;
-    vlc_mutex_unlock( p_sub->p_lock );
-
-    if( p_sub->i_start != i_stop )
+    switch( p_sys->i_mode )
     {
-        /* Append all messages to log file */
-        for( i_start = p_sub->i_start;
-             i_start != i_stop;
-             i_start = (i_start+1) % VLC_MSG_QSIZE )
-        {
-            switch( p_sub->p_msg[i_start].i_type )
-            {
-            case VLC_MSG_ERR:
-                if( i_verbose < 0 ) continue;
-                break;
-            case VLC_MSG_INFO:
-                if( i_verbose < 0 ) continue;
-                break;
-            case VLC_MSG_WARN:
-                if( i_verbose < 1 ) continue;
-                break;
-            case VLC_MSG_DBG:
-                if( i_verbose < 2 ) continue;
-                break;
-            }
-
-            switch( i_mode )
-            {
-            case MODE_HTML:
-                HtmlPrint( &p_sub->p_msg[i_start], p_file );
-                break;
+        case MODE_HTML:
+            HtmlPrint( p_item, p_sys->p_file );
+            break;
 #ifdef HAVE_SYSLOG_H
-            case MODE_SYSLOG:
-                SyslogPrint( &p_sub->p_msg[i_start] );
-                break;
+        case MODE_SYSLOG:
+            SyslogPrint( p_item );
+            break;
 #endif
-            case MODE_TEXT:
-            default:
-                TextPrint( &p_sub->p_msg[i_start], p_file );
-                break;
-            }
-        }
-
-        vlc_mutex_lock( p_sub->p_lock );
-        p_sub->i_start = i_start;
-        vlc_mutex_unlock( p_sub->p_lock );
+        case MODE_TEXT:
+        default:
+            TextPrint( p_item, p_sys->p_file );
+            break;
     }
 }
 
-static const char *ppsz_type[4] = { ": ", " error: ",
-                                    " warning: ", " debug: " };
+static const char ppsz_type[4][11] = {
+    ": ",
+    " error: ",
+    " warning: ",
+    " debug: ",
+};
 
 static void TextPrint( const msg_item_t *p_msg, FILE *p_file )
 {
-    LOG_STRING( p_msg->psz_module, p_file );
-    LOG_STRING( ppsz_type[p_msg->i_type], p_file );
-    LOG_STRING( p_msg->psz_msg, p_file );
-    LOG_STRING( "\n", p_file );
+    fprintf( p_file, "%s%s%s\n", p_msg->psz_module, ppsz_type[p_msg->i_type],
+             p_msg->psz_msg );
 }
 
 #ifdef HAVE_SYSLOG_H
@@ -409,11 +348,10 @@ static void SyslogPrint( const msg_item_t *p_msg )
     int i_priority = i_prio[p_msg->i_type];
 
     if( p_msg->psz_header )
-        syslog( i_priority, "%s%s %s: %s", p_msg->psz_header,
-                ppsz_type[p_msg->i_type],
-                p_msg->psz_module, p_msg->psz_msg );
+        syslog( i_priority, "%s %s%s%s", p_msg->psz_header, p_msg->psz_module,
+                ppsz_type[p_msg->i_type], p_msg->psz_msg );
     else
-        syslog( i_priority, "%s%s: %s", p_msg->psz_module, 
+        syslog( i_priority, "%s%s%s", p_msg->psz_module, 
                 ppsz_type[p_msg->i_type], p_msg->psz_msg );
  
 }
@@ -421,40 +359,14 @@ static void SyslogPrint( const msg_item_t *p_msg )
 
 static void HtmlPrint( const msg_item_t *p_msg, FILE *p_file )
 {
-    static const char *ppsz_color[4] = { "<span style=\"color: #ffffff\">",
-                                         "<span style=\"color: #ff6666\">",
-                                         "<span style=\"color: #ffff66\">",
-                                         "<span style=\"color: #aaaaaa\">" };
+    static const char ppsz_color[4][30] = {
+        "<span style=\"color: #ffffff\">",
+        "<span style=\"color: #ff6666\">",
+        "<span style=\"color: #ffff66\">",
+        "<span style=\"color: #aaaaaa\">",
+    };
 
-    LOG_STRING( p_msg->psz_module, p_file );
-    LOG_STRING( ppsz_type[p_msg->i_type], p_file );
-    LOG_STRING( ppsz_color[p_msg->i_type], p_file );
-    LOG_STRING( p_msg->psz_msg, p_file );
-    LOG_STRING( "</span>\n", p_file );
-}
-
-static void DoRRD( intf_thread_t *p_intf )
-{
-    mtime_t now = mdate();
-    if( now - p_intf->p_sys->last_update < 1000000 )
-        return;
-    p_intf->p_sys->last_update = now;
-
-    if( p_intf->p_libvlc->p_stats )
-    {
-        time(&p_intf->p_sys->now);
-        lldiv_t din = lldiv( p_intf->p_libvlc->p_stats->f_input_bitrate * 1000000,
-                             1000 );
-        lldiv_t ddm = lldiv( p_intf->p_libvlc->p_stats->f_demux_bitrate * 1000000,
-                             1000 );
-        lldiv_t dout = lldiv( p_intf->p_libvlc->p_stats->f_output_bitrate * 1000000,
-                             1000 );
-        fprintf( p_intf->p_sys->p_rrd,
-                   "%"PRIi64":%lld.%03u:%lld.%03u:%lld.%03u\n",
-                   (uintmax_t)p_intf->p_sys->now,
-                   din.quot, (unsigned int)din.rem,
-                   ddm.quot, (unsigned int)ddm.rem,
-                   dout.quot, (unsigned int)dout.rem );
-        fflush( p_intf->p_sys->p_rrd );
-    }
+    fprintf( p_file, "%s%s%s%s</span>\n", p_msg->psz_module,
+             ppsz_type[p_msg->i_type], ppsz_color[p_msg->i_type],
+             p_msg->psz_msg );
 }
