@@ -2,7 +2,7 @@
  * bluescreen.c : Bluescreen (weather channel like) video filter for vlc
  *****************************************************************************
  * Copyright (C) 2005-2007 the VideoLAN team
- * $Id: 2499f55d9b016180d56d29c1a85010dd8182f00e $
+ * $Id: 90396c81cc447b473d6fddf74300c1a4acc1aa35 $
  *
  * Authors: Antoine Cellerier <dionoea at videolan tod org>
  *
@@ -38,8 +38,8 @@
 #define BLUESCREEN_HELP N_( \
     "This effect, also known as \"greenscreen\" or \"chroma key\" blends " \
     "the \"blue parts\" of the foreground image of the mosaic on the " \
-    "background (like weather forcasts). You can choose the \"key\" " \
-    "color for blending (blyyue by default)." )
+    "background (like weather forecasts). You can choose the \"key\" " \
+    "color for blending (blue by default)." )
 
 #define BLUESCREENU_TEXT N_("Bluescreen U value")
 #define BLUESCREENU_LONGTEXT N_( \
@@ -75,27 +75,27 @@ static int BluescreenCallback( vlc_object_t *, char const *,
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-vlc_module_begin();
-    set_description( N_("Bluescreen video filter") );
-    set_shortname( N_("Bluescreen" ));
-    set_help( BLUESCREEN_HELP );
-    set_category( CAT_VIDEO );
-    set_subcategory( SUBCAT_VIDEO_VFILTER );
-    set_capability( "video filter2", 0 );
-    add_shortcut( "bluescreen" );
-    set_callbacks( Create, Destroy );
+vlc_module_begin ()
+    set_description( N_("Bluescreen video filter") )
+    set_shortname( N_("Bluescreen" ))
+    set_help( BLUESCREEN_HELP )
+    set_category( CAT_VIDEO )
+    set_subcategory( SUBCAT_VIDEO_VFILTER )
+    set_capability( "video filter2", 0 )
+    add_shortcut( "bluescreen" )
+    set_callbacks( Create, Destroy )
 
     add_integer_with_range( CFG_PREFIX "u", 120, 0, 255, NULL,
-                            BLUESCREENU_TEXT, BLUESCREENU_LONGTEXT, false );
+                            BLUESCREENU_TEXT, BLUESCREENU_LONGTEXT, false )
     add_integer_with_range( CFG_PREFIX "v", 90, 0, 255, NULL,
-                            BLUESCREENV_TEXT, BLUESCREENV_LONGTEXT, false );
+                            BLUESCREENV_TEXT, BLUESCREENV_LONGTEXT, false )
     add_integer_with_range( CFG_PREFIX "ut", 17, 0, 255, NULL,
                             BLUESCREENUTOL_TEXT, BLUESCREENUTOL_LONGTEXT,
-                            false );
+                            false )
     add_integer_with_range( CFG_PREFIX "vt", 17, 0, 255, NULL,
                             BLUESCREENVTOL_TEXT, BLUESCREENVTOL_LONGTEXT,
-                            false );
-vlc_module_end();
+                            false )
+vlc_module_end ()
 
 static const char *const ppsz_filter_options[] = {
     "u", "v", "ut", "vt", NULL
@@ -103,6 +103,7 @@ static const char *const ppsz_filter_options[] = {
 
 struct filter_sys_t
 {
+    vlc_mutex_t lock;
     int i_u, i_v, i_ut, i_vt;
     uint8_t *p_at;
 };
@@ -130,9 +131,11 @@ static int Create( vlc_object_t *p_this )
     config_ChainParse( p_filter, CFG_PREFIX, ppsz_filter_options,
                        p_filter->p_cfg );
 
+    int val;
+    vlc_mutex_init( &p_sys->lock );
 #define GET_VAR( name, min, max )                                           \
-    p_sys->i_##name = __MIN( max, __MAX( min,                               \
-        var_CreateGetIntegerCommand( p_filter, CFG_PREFIX #name ) ) );      \
+    val = var_CreateGetIntegerCommand( p_filter, CFG_PREFIX #name );        \
+    p_sys->i_##name = __MIN( max, __MAX( min, val ) );                      \
     var_AddCallback( p_filter, CFG_PREFIX #name, BluescreenCallback, p_sys );
 
     GET_VAR( u, 0x00, 0xff );
@@ -140,6 +143,7 @@ static int Create( vlc_object_t *p_this )
     GET_VAR( ut, 0x00, 0xff );
     GET_VAR( vt, 0x00, 0xff );
     p_sys->p_at = NULL;
+#undef GET_VAR
 
     p_filter->pf_video_filter = Filter;
 
@@ -149,9 +153,16 @@ static int Create( vlc_object_t *p_this )
 static void Destroy( vlc_object_t *p_this )
 {
     filter_t *p_filter = (filter_t *)p_this;
+    filter_sys_t *p_sys = p_filter->p_sys;
 
-    free( p_filter->p_sys->p_at );
-    free( p_filter->p_sys );
+    var_DelCallback( p_filter, CFG_PREFIX "u", BluescreenCallback, p_sys );
+    var_DelCallback( p_filter, CFG_PREFIX "v", BluescreenCallback, p_sys );
+    var_DelCallback( p_filter, CFG_PREFIX "ut", BluescreenCallback, p_sys );
+    var_DelCallback( p_filter, CFG_PREFIX "vt", BluescreenCallback, p_sys );
+
+    free( p_sys->p_at );
+    vlc_mutex_destroy( &p_sys->lock );
+    free( p_sys );
 }
 
 static picture_t *Filter( filter_t *p_filter, picture_t *p_pic )
@@ -179,10 +190,12 @@ static picture_t *Filter( filter_t *p_filter, picture_t *p_pic )
     p_sys->p_at = realloc( p_sys->p_at, i_lines * i_pitch * sizeof( uint8_t ) );
     p_at = p_sys->p_at;
 
+    vlc_mutex_lock( &p_sys->lock );
     umin = p_sys->i_u - p_sys->i_ut >= 0x00 ? p_sys->i_u - p_sys->i_ut : 0x00;
     umax = p_sys->i_u + p_sys->i_ut <= 0xff ? p_sys->i_u + p_sys->i_ut : 0xff;
     vmin = p_sys->i_v - p_sys->i_vt >= 0x00 ? p_sys->i_v - p_sys->i_vt : 0x00;
     vmax = p_sys->i_v + p_sys->i_vt <= 0xff ? p_sys->i_v + p_sys->i_vt : 0xff;
+    vlc_mutex_unlock( &p_sys->lock );
 
     for( i = 0; i < i_lines*i_pitch; i++ )
     {
@@ -252,23 +265,17 @@ static int BluescreenCallback( vlc_object_t *p_this, char const *psz_var,
     VLC_UNUSED(p_this); VLC_UNUSED(oldval);
     filter_sys_t *p_sys = (filter_sys_t *) p_data;
 
+    vlc_mutex_lock( &p_sys->lock );
 #define VAR_IS( a ) !strcmp( psz_var, CFG_PREFIX a )
     if( VAR_IS( "u" ) )
-    {
         p_sys->i_u = __MAX( 0, __MIN( 255, newval.i_int ) );
-    }
     else if( VAR_IS( "v" ) )
-    {
         p_sys->i_v = __MAX( 0, __MIN( 255, newval.i_int ) );
-    }
     else if( VAR_IS( "ut" ) )
-    {
         p_sys->i_ut = __MAX( 0, __MIN( 255, newval.i_int ) );
-    }
     else if( VAR_IS( "vt" ) )
-    {
         p_sys->i_vt = __MAX( 0, __MIN( 255, newval.i_int ) );
-    }
+    vlc_mutex_unlock( &p_sys->lock );
 
     return VLC_SUCCESS;
 }
