@@ -2,7 +2,7 @@
  * erase.c : logo erase video filter
  *****************************************************************************
  * Copyright (C) 2007 the VideoLAN team
- * $Id$
+ * $Id: 982858473b4652d2d77cd478aa6ed5c35e27b46a $
  *
  * Authors: Antoine Cellerier <dionoea -at- videolan -dot- org>
  *
@@ -62,21 +62,21 @@ static int EraseCallback( vlc_object_t *, char const *,
 
 #define CFG_PREFIX "erase-"
 
-vlc_module_begin();
-    set_description( N_("Erase video filter") );
-    set_shortname( N_( "Erase" ));
-    set_capability( "video filter2", 0 );
-    set_category( CAT_VIDEO );
-    set_subcategory( SUBCAT_VIDEO_VFILTER );
+vlc_module_begin ()
+    set_description( N_("Erase video filter") )
+    set_shortname( N_( "Erase" ))
+    set_capability( "video filter2", 0 )
+    set_category( CAT_VIDEO )
+    set_subcategory( SUBCAT_VIDEO_VFILTER )
 
     add_file( CFG_PREFIX "mask", NULL, NULL,
-              MASK_TEXT, MASK_LONGTEXT, false );
-    add_integer( CFG_PREFIX "x", 0, NULL, POSX_TEXT, POSX_LONGTEXT, false );
-    add_integer( CFG_PREFIX "y", 0, NULL, POSY_TEXT, POSY_LONGTEXT, false );
+              MASK_TEXT, MASK_LONGTEXT, false )
+    add_integer( CFG_PREFIX "x", 0, NULL, POSX_TEXT, POSX_LONGTEXT, false )
+    add_integer( CFG_PREFIX "y", 0, NULL, POSY_TEXT, POSY_LONGTEXT, false )
 
-    add_shortcut( "erase" );
-    set_callbacks( Create, Destroy );
-vlc_module_end();
+    add_shortcut( "erase" )
+    set_callbacks( Create, Destroy )
+vlc_module_end ()
 
 static const char *const ppsz_filter_options[] = {
     "mask", "x", "y", NULL
@@ -114,6 +114,9 @@ static void LoadMask( filter_t *p_filter, const char *psz_filename )
         p_filter->p_sys->p_mask = p_old_mask;
         msg_Err( p_filter, "Error while loading new mask. Keeping old mask." );
     }
+    else
+        msg_Err( p_filter, "Error while loading new mask. No mask available." );
+
     image_HandlerDelete( p_image );
 }
 
@@ -160,21 +163,22 @@ static int Create( vlc_object_t *p_this )
     if( !psz_filename )
     {
         msg_Err( p_filter, "Missing 'mask' option value." );
+        free( p_sys );
         return VLC_EGENERIC;
     }
 
     p_sys->p_mask = NULL;
     LoadMask( p_filter, psz_filename );
     free( psz_filename );
+
     p_sys->i_x = var_CreateGetIntegerCommand( p_filter, CFG_PREFIX "x" );
     p_sys->i_y = var_CreateGetIntegerCommand( p_filter, CFG_PREFIX "y" );
 
+    vlc_mutex_init( &p_sys->lock );
     var_AddCallback( p_filter, CFG_PREFIX "x", EraseCallback, p_sys );
     var_AddCallback( p_filter, CFG_PREFIX "y", EraseCallback, p_sys );
     var_AddCallback( p_filter, CFG_PREFIX "mask", EraseCallback, p_sys );
 
-    vlc_mutex_init( &p_sys->lock );
-
     return VLC_SUCCESS;
 }
 
@@ -188,6 +192,9 @@ static void Destroy( vlc_object_t *p_this )
     if( p_sys->p_mask )
         picture_Release( p_sys->p_mask );
 
+    var_DelCallback( p_filter, CFG_PREFIX "x", EraseCallback, p_sys );
+    var_DelCallback( p_filter, CFG_PREFIX "y", EraseCallback, p_sys );
+    var_DelCallback( p_filter, CFG_PREFIX "mask", EraseCallback, p_sys );
     vlc_mutex_destroy( &p_sys->lock );
 
     free( p_filter->p_sys );
@@ -199,6 +206,7 @@ static void Destroy( vlc_object_t *p_this )
 static picture_t *Filter( filter_t *p_filter, picture_t *p_pic )
 {
     picture_t *p_outpic;
+    filter_sys_t *p_sys = p_filter->p_sys;
 
     if( !p_pic ) return NULL;
 
@@ -209,8 +217,13 @@ static picture_t *Filter( filter_t *p_filter, picture_t *p_pic )
         return NULL;
     }
 
-    /* Here */
-    FilterErase( p_filter, p_pic, p_outpic );
+    /* If the mask is empty: just copy the image */
+    vlc_mutex_lock( &p_sys->lock );
+    if( p_sys->p_mask )
+        FilterErase( p_filter, p_pic, p_outpic );
+    else
+        picture_CopyPixels( p_outpic, p_pic );
+    vlc_mutex_unlock( &p_sys->lock );
 
     return CopyInfoAndRelease( p_outpic, p_pic );
 }
@@ -223,13 +236,11 @@ static void FilterErase( filter_t *p_filter, picture_t *p_inpic,
 {
     filter_sys_t *p_sys = p_filter->p_sys;
 
-    int i_plane;
-
     const int i_mask_pitch = p_sys->p_mask->A_PITCH;
     const int i_mask_visible_pitch = p_sys->p_mask->p[A_PLANE].i_visible_pitch;
     const int i_mask_visible_lines = p_sys->p_mask->p[A_PLANE].i_visible_lines;
 
-    for( i_plane = 0; i_plane < p_inpic->i_planes; i_plane++ )
+    for( int i_plane = 0; i_plane < p_inpic->i_planes; i_plane++ )
     {
         const int i_pitch = p_inpic->p[i_plane].i_pitch;
         const int i_2pitch = i_pitch<<1;
@@ -240,9 +251,8 @@ static void FilterErase( filter_t *p_filter, picture_t *p_inpic,
         uint8_t *p_inpix = p_inpic->p[i_plane].p_pixels;
         uint8_t *p_outpix = p_outpic->p[i_plane].p_pixels;
         uint8_t *p_mask = p_sys->p_mask->A_PIXELS;
+        int i_x = p_sys->i_x, i_y = p_sys->i_y;
 
-        int i_x = p_sys->i_x,
-            i_y = p_sys->i_y;
         int x, y;
         int i_height = i_mask_visible_lines;
         int i_width  = i_mask_visible_pitch;
