@@ -2,7 +2,7 @@
  * direct3d.c: Windows Direct3D video output module
  *****************************************************************************
  * Copyright (C) 2006-2009 the VideoLAN team
- *$Id: e2040afe09fa71bc4a6f27c071e3988e1bf48858 $
+ *$Id: 8951bbe1cfc0230d0e9f0804434b5ec0434a9897 $
  *
  * Authors: Damien Fouilleul <damienf@videolan.org>
  *
@@ -723,6 +723,16 @@ static int Direct3DFillPresentationParameters(vout_thread_t *p_vout)
     d3dpp->BackBufferCount        = 1;
     d3dpp->EnableAutoDepthStencil = FALSE;
 
+    const unsigned i_adapter_count = IDirect3D9_GetAdapterCount(p_d3dobj);
+    for( unsigned i = 1; i < i_adapter_count; i++ )
+    {
+        hr = IDirect3D9_GetAdapterDisplayMode(p_d3dobj, i, &d3ddm );
+        if( FAILED(hr) )
+            continue;
+        d3dpp->BackBufferWidth  = __MAX(d3dpp->BackBufferWidth,  d3ddm.Width);
+        d3dpp->BackBufferHeight = __MAX(d3dpp->BackBufferHeight, d3ddm.Height);
+    }
+
     RECT *display = &p_vout->p_sys->rect_display;
     display->left   = 0;
     display->top    = 0;
