@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vout.c: Windows DirectX video output display method
  *****************************************************************************
- * Copyright (C) 2001-2004 the VideoLAN team
- * $Id: a38a8d1612e62d546b07af376bc1c10e4c9f9bdc $
+ * Copyright (C) 2001-2009 the VideoLAN team
+ * $Id: cbb7eafca17135ba238cbd2fa744770c630eb6a0 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -47,6 +47,7 @@
 #include <vlc_vout.h>
 
 #include <windows.h>
+#include <tchar.h>
 #include <ddraw.h>
 #include <commctrl.h>
 
@@ -160,33 +161,33 @@ static int WallpaperCallback( vlc_object_t *, char const *,
 static const char *const ppsz_dev[] = { "" };
 static const char *const ppsz_dev_text[] = { N_("Default") };
 
-vlc_module_begin();
-    set_shortname( "DirectX" );
-    set_category( CAT_VIDEO );
-    set_subcategory( SUBCAT_VIDEO_VOUT );
+vlc_module_begin ()
+    set_shortname( "DirectX" )
+    set_category( CAT_VIDEO )
+    set_subcategory( SUBCAT_VIDEO_VOUT )
     add_bool( "directx-hw-yuv", 1, NULL, HW_YUV_TEXT, HW_YUV_LONGTEXT,
-              true );
+              true )
     add_bool( "directx-use-sysmem", 0, NULL, SYSMEM_TEXT, SYSMEM_LONGTEXT,
-              true );
+              true )
     add_bool( "directx-3buffering", 1, NULL, TRIPLEBUF_TEXT,
-              TRIPLEBUF_LONGTEXT, true );
+              TRIPLEBUF_LONGTEXT, true )
 
     add_string( "directx-device", "", NULL, DEVICE_TEXT, DEVICE_LONGTEXT,
-                true );
-        change_string_list( ppsz_dev, ppsz_dev_text, FindDevicesCallback );
-        change_action_add( FindDevicesCallback, N_("Refresh list") );
+                true )
+        change_string_list( ppsz_dev, ppsz_dev_text, FindDevicesCallback )
+        change_action_add( FindDevicesCallback, N_("Refresh list") )
 
     add_bool( "directx-wallpaper", 0, NULL, WALLPAPER_TEXT, WALLPAPER_LONGTEXT,
-              true );
+              true )
 
-    set_description( N_("DirectX video output") );
-    set_capability( "video output", 100 );
-    add_shortcut( "directx" );
-    set_callbacks( OpenVideo, CloseVideo );
+    set_description( N_("DirectX video output") )
+    set_capability( "video output", 100 )
+    add_shortcut( "directx" )
+    set_callbacks( OpenVideo, CloseVideo )
 
     /* FIXME: Hack to avoid unregistering our window class */
-    linked_with_a_crap_library_which_uses_atexit( );
-vlc_module_end();
+    linked_with_a_crap_library_which_uses_atexit ()
+vlc_module_end ()
 
 #if 0 /* FIXME */
     /* check if we registered a window class because we need to
@@ -208,10 +209,9 @@ static int OpenVideo( vlc_object_t *p_this )
     HMODULE huser32;
 
     /* Allocate structure */
-    p_vout->p_sys = malloc( sizeof( vout_sys_t ) );
+    p_vout->p_sys = calloc( 1, sizeof( vout_sys_t ) );
     if( p_vout->p_sys == NULL )
         return VLC_ENOMEM;
-    memset( p_vout->p_sys, 0, sizeof( vout_sys_t ) );
 
     /* Initialisations */
     p_vout->pf_init = Init;
@@ -277,14 +277,18 @@ static int OpenVideo( vlc_object_t *p_this )
     p_vout->p_sys->p_event =
         vlc_object_create( p_vout, sizeof(event_thread_t) );
     p_vout->p_sys->p_event->p_vout = p_vout;
+    p_vout->p_sys->p_event->window_ready = CreateEvent( NULL, TRUE, FALSE, NULL );
     if( vlc_thread_create( p_vout->p_sys->p_event, "Vout Events Thread",
-                           EventThread, 0, 1 ) )
+                           EventThread, 0 ) )
     {
         msg_Err( p_vout, "cannot create Vout EventThread" );
+        CloseHandle( p_vout->p_sys->p_event->window_ready );
         vlc_object_release( p_vout->p_sys->p_event );
         p_vout->p_sys->p_event = NULL;
         goto error;
     }
+    WaitForSingleObject( p_vout->p_sys->p_event->window_ready, INFINITE );
+    CloseHandle( p_vout->p_sys->p_event->window_ready );
 
     if( p_vout->p_sys->p_event->b_error )
     {
@@ -312,8 +316,7 @@ static int OpenVideo( vlc_object_t *p_this )
 
     /* Variable to indicate if the window should be on top of others */
     /* Trigger a callback right now */
-    var_Get( p_vout, "video-on-top", &val );
-    var_Set( p_vout, "video-on-top", val );
+    var_TriggerCallback( p_vout, "video-on-top" );
 
     /* Variable to indicate if the window should be on top of others */
     /* Trigger a callback right now */
@@ -321,15 +324,13 @@ static int OpenVideo( vlc_object_t *p_this )
     val.psz_string = _("Wallpaper");
     var_Change( p_vout, "directx-wallpaper", VLC_VAR_SETTEXT, &val, NULL );
     var_AddCallback( p_vout, "directx-wallpaper", WallpaperCallback, NULL );
-    var_Get( p_vout, "directx-wallpaper", &val );
-    var_Set( p_vout, "directx-wallpaper", val );
+    var_TriggerCallback( p_vout, "directx-wallpaper" );
 
     /* disable screensaver by temporarily changing system settings */
     p_vout->p_sys->i_spi_lowpowertimeout = 0;
     p_vout->p_sys->i_spi_powerofftimeout = 0;
     p_vout->p_sys->i_spi_screensavetimeout = 0;
-    var_Get( p_vout, "disable-screensaver", &val);
-    if( val.b_bool ) {
+    if( var_GetBool( p_vout, "disable-screensaver" ) ) {
         msg_Dbg(p_vout, "disabling screen saver");
         SystemParametersInfo(SPI_GETLOWPOWERTIMEOUT,
             0, &(p_vout->p_sys->i_spi_lowpowertimeout), 0);
@@ -364,17 +365,12 @@ static int OpenVideo( vlc_object_t *p_this )
 static int Init( vout_thread_t *p_vout )
 {
     int i_chroma_backup;
-    vlc_value_t val;
 
     /* Get a few default parameters */
-    var_Get( p_vout, "overlay", &val );
-    p_vout->p_sys->b_using_overlay = val.b_bool;
-    var_Get( p_vout, "directx-use-sysmem", &val );
-    p_vout->p_sys->b_use_sysmem = val.b_bool;
-    var_Get( p_vout, "directx-hw-yuv", &val );
-    p_vout->p_sys->b_hw_yuv = val.b_bool;
-    var_Get( p_vout, "directx-3buffering", &val );
-    p_vout->p_sys->b_3buf_overlay = val.b_bool;
+    p_vout->p_sys->b_using_overlay = var_GetBool( p_vout, "overlay" );
+    p_vout->p_sys->b_use_sysmem = var_GetBool( p_vout, "directx-use-sysmem" );
+    p_vout->p_sys->b_hw_yuv = var_GetBool( p_vout, "directx-hw-yuv" );
+    p_vout->p_sys->b_3buf_overlay = var_GetBool( p_vout, "directx-3buffering" );
 
     /* Initialise DirectDraw if not already done.
      * We do this here because on multi-monitor systems we may have to
@@ -453,6 +449,7 @@ static int Init( vout_thread_t *p_vout )
         /* If it still didn't work then don't try to use an overlay */
         p_vout->output.i_chroma = i_chroma_backup;
         p_vout->p_sys->b_using_overlay = 0;
+        msg_Warn( p_vout, "Could not initialize directx overlay" ) ;
         NewPictureVec( p_vout, p_vout->p_picture, MAX_DIRECTBUFFERS );
     }
 
@@ -517,6 +514,7 @@ static void CloseVideo( vlc_object_t *p_this )
     vlc_mutex_destroy( &p_vout->p_sys->lock );
 
     /* Make sure the wallpaper is restored */
+    var_DelCallback( p_vout, "directx-wallpaper", WallpaperCallback, NULL );
     SwitchWallpaperMode( p_vout, false );
 
     /* restore screensaver system settings */
@@ -598,6 +596,28 @@ static int Manage( vout_thread_t *p_vout )
         }
     }
 
+    /* autoscale toggle */
+    if( p_vout->i_changes & VOUT_SCALE_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_SCALE_CHANGE;
+
+        p_vout->b_autoscale = var_GetBool( p_vout, "autoscale" );
+        p_vout->i_zoom = (int) ZOOM_FP_FACTOR;
+
+        UpdateRects( p_vout, true );
+    }
+
+    /* scaling factor */
+    if( p_vout->i_changes & VOUT_ZOOM_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_ZOOM_CHANGE;
+
+        p_vout->b_autoscale = false;
+        p_vout->i_zoom =
+            (int)( ZOOM_FP_FACTOR * var_GetFloat( p_vout, "scale" ) );
+        UpdateRects( p_vout, true );
+    }
+
     /* Check for cropping / aspect changes */
     if( p_vout->i_changes & VOUT_CROP_CHANGE ||
         p_vout->i_changes & VOUT_ASPECT_CHANGE )
@@ -2129,7 +2149,7 @@ static int WallpaperCallback( vlc_object_t *p_this, char const *psz_cmd,
     if( (newval.b_bool && !p_vout->p_sys->b_wallpaper) ||
         (!newval.b_bool && p_vout->p_sys->b_wallpaper) )
     {
-        playlist_t *p_playlist = pl_Yield( p_vout );
+        playlist_t *p_playlist = pl_Hold( p_vout );
 
         if( p_playlist )
         {
