@@ -2,7 +2,7 @@
  * item.c: input_item management
  *****************************************************************************
  * Copyright (C) 1998-2004 the VideoLAN team
- * $Id: c8158afdaf2c89714c01fa8fde7de2ec6de86c8e $
+ * $Id: 7349d9e05bce853ce87f0046373fc8d078d8f41f $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *
@@ -712,7 +712,6 @@ void input_item_SetEpg( input_item_t *p_item,
 {
     input_item_DelInfo( p_item, psz_epg, NULL );
 
-#ifdef HAVE_LOCALTIME_R
     vlc_mutex_lock( &p_item->lock );
     for( int i = 0; i < p_epg->i_event; i++ )
     {
@@ -727,10 +726,11 @@ void input_item_SetEpg( input_item_t *p_item,
                   1900 + tm_start.tm_year, 1 + tm_start.tm_mon, tm_start.tm_mday,
                   tm_start.tm_hour, tm_start.tm_min, tm_start.tm_sec );
         if( p_evt->psz_short_description || p_evt->psz_description )
-            InputItemAddInfo( p_item, psz_epg, psz_start, "%s (%2.2d:%2.2d) - %s",
+            InputItemAddInfo( p_item, psz_epg, psz_start, "%s (%2.2d:%2.2d) - %s %s",
                               p_evt->psz_name,
                               p_evt->i_duration/60/60, (p_evt->i_duration/60)%60,
-                              p_evt->psz_short_description ? p_evt->psz_short_description : p_evt->psz_description );
+                              p_evt->psz_short_description ? p_evt->psz_short_description : "" ,
+                              p_evt->psz_description ? p_evt->psz_description : "" );
         else
             InputItemAddInfo( p_item, psz_epg, psz_start, "%s (%2.2d:%2.2d)",
                               p_evt->psz_name,
@@ -745,7 +745,6 @@ void input_item_SetEpg( input_item_t *p_item,
         event.type = vlc_InputItemInfoChanged;
         vlc_event_send( &p_item->event_manager, &event );
     }
-#endif
 }
 
 
